#!/usr/bin/perl
###########################################################################
#    WWWThreads  - Interactive Discussion Forum
#    Rick Baker scream@screamingweb.com 
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
###########################################################################

  use wwwthreads qw(%config $dbh);

  use DBI;
  use strict;


# Get the input
  my %FORM = wwwthreads::get_input;


# Assign the variables
  my $Username = $FORM{'Username'};
  my $Email = $FORM{'Email'};
  my $Password = $FORM{'Password'};
  my $Verify = $FORM{'Verify'};


# If all required info is not filled in, then we can't proceed 
  if((!$Username)||(!$Email)||(!$Password)||(!$Verify)){

    wwwthreads::not_right("All of the required information has not been filled in.  Please try again.");

  }


# If the Username isn't the proper length then we can't proceed  
  if((length($Username) >16) || (length($Username) < 3)) {

    wwwthreads::not_right("Username is not the proper length.  Please try again.");

  }


# If the Password isn't the proper length then we can't proceed
  if((length($Password) > 10) || (length($Password) < 4)) {

    wwwthreads::not_right("Password is not the proper length.  Please try again.");

  }


# Check that the Password matches the Verify password
  if($Password ne $Verify) {

    wwwthreads::not_right("Passwords do not match.  Please try again.");

  }


# Connect to the database
  $dbh = wwwthreads::db_connect;


# Check to make sure the Username is available
  my $Username_q = $dbh -> quote ($Username);
  my $query =qq!
    SELECT Username
    FROM Users
    WHERE Username = $Username_q
  !;
  my $sth = $dbh -> prepare ($query) or die "Query syntax error: $DBI::errstr. Query: $query";
  $sth -> execute() or die "Can't execute query: $query. Reason: $DBI::errstr";
  my $rows = $sth -> rows;
  
# If sql returned a row then that username is already in use.
  if( $rows > 0 ) {

    wwwthreads::not_right("That Username is already in use.  Please try again.");

  }


# Check to see if this is the first user
  $query = qq!
    SELECT Username
    FROM Users
  !;
  
  $sth = $dbh -> prepare ($query) or die "Query syntax error: $DBI::errstr. Query: $query";
  $sth -> execute() or die "Can't execute query: $query. Reason: $DBI::errstr";
  $rows = $sth -> rows;

  my $Status = "";
# If this is the first user, then status is Administrator
  if ($rows < 1){

    $Status = "Administrator";

  } else {

    # If not then status is User
    $Status = "User";

  } 


# Get the date
  my $date = wwwthreads::get_date;


# Put the user into the database
  my $Status_q   = $dbh -> quote($Status);
  $Username_q    = $dbh -> quote($Username);
  my $Password_q = $dbh -> quote($Password);
  my $Email_q    = $dbh -> quote($Email);
  my $Display_q  = $dbh -> quote($wwwthreads::config{'postlist'});
  my $View_q     = $dbh -> quote($wwwthreads::config{'threaded'}); 
  $query = qq!
    INSERT INTO Users (Username,Password,Email,Totalposts,Laston,Status,Sort,Display,View,PostsPer)
    VALUES ($Username_q,$Password_q,$Email_q,0,$date,$Status_q,$wwwthreads::config{'sort'},$Display_q,$View_q,$wwwthreads::config{'postsperpage'})
  !;
  my $rc = $dbh -> do($query) or die "Can't execute query: $query. Reason: $DBI::errstr";


# Grab welcome.include  
  open(FILE,"$wwwthreads::config{'includes'}/welcome.include") or die "Can't open file '$wwwthreads::config{'includes'}/welcome.include'";
  my $Message = '';
  while(<FILE>){
    $Message = $Message."$_";
  }
  my $Message_q = $dbh -> quote($Message);
  close(FILE);


# Get the Administrator's Username
  my $Admin = "Administrator";
  my $Admin_q = $dbh -> quote($Admin);
  
  $query = qq!
    SELECT Username
    FROM Users
    WHERE Status = $Admin_q
  !;
  $sth = $dbh -> prepare ($query) or die "Query syntax error: $DBI::errstr. Query: $query";
  $sth -> execute() or die "Can't execute query: $query. Reason: $DBI::errstr";

  my ($Sender) = $sth -> fetchrow_array;
  my $Sender_q = $dbh -> quote ($Sender);


# Get the next message number
  $query = qq!
    SELECT _seq FROM Messages
  !;
  $sth = $dbh -> prepare ($query) or die "Query syntax error: $DBI::errstr. Query: $query";
  $sth -> execute() or die "Can't execute query: $query. Reason: $DBI::errstr";

  my ($Number) = $sth -> fetchrow_array;

  my $Messstat = "N";
  my $Messstat_q = $dbh -> quote($Messstat);
  my $Subject = "Welcome";
  my $Subject_q = $dbh -> quote($Subject);

# Put the message into the Messages table
  $query = qq!
    INSERT INTO Messages (Username,Read,Subject,Sender,Message,When,Number)
    VALUES ($Username_q,$Messstat_q,$Subject_q,$Sender_q,$Message_q,$date,$Number)
  !;
  $rc = $dbh -> do($query) or die "Can't execute query: $query. Reason: $DBI::errstr";

# Set their default cookies
  wwwthreads::set_cookie('Sort',$wwwthreads::config{'sort'},1,'Display',$wwwthreads::config{'postlist'},1,'View',$wwwthreads::config{'threaded'},1,'PostsPer',$wwwthreads::config{'postsperpage'},1);


# Send them a confirmation
  wwwthreads::send_header("Username has been registered");
  wwwthreads::table_header("Username has been registered");

  print qq~ 

  The Username '$Username' has been reserved.  You may now <a href="$wwwthreads::config{'cgiurl'}/login.pl">Login</a> and post with this Usename.  Thanks for registering a Username.  Please take the time to edit your profile before posting on the forums.  Editing your profile will allow you to customize the layout of the forums to suit your particular tastes and needs.  
  </P>

  ~;

# Send a footer
  $sth -> finish;
  wwwthreads::send_footer;

