#!/usr/bin/perl 
###########################################################################
#    WWWThreads  - Interactive Discussion Forum
#    Rick Baker scream@screamingweb.com 
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
###########################################################################

  use wwwthreads qw(%config $dbh);
  use DBI;
  use strict;


# Get the input
  my %FORM = wwwthreads::get_input();


# Assign the variables
  my $Title       = $FORM{'Title'};
  my $Keyword     = $FORM{'Keyword'};
  my $Description = $FORM{'Description'};
  my $HTML        = $FORM{'HTML'};
  my $Anonymous   = $FORM{'Anonymous'};
  my $Locked      = $FORM{'Locked'};


# If they didn't check the HTML or Anonymous box then default to Off
  $HTML ||= "Off";

  $Anonymous ||= "Off";


# If they didn't checked the Locked then default to open

  $Locked ||= "Open";


# Grab the cookie
  my %cookie = wwwthreads::get_cookie();


# Assign the variables
  my $Username = $cookie{'Username'};
  my $Password = $cookie{'Password'};


# Connect to the database
  wwwthreads::db_connect();


# Make sure they are an admin
  wwwthreads::validate_admin("$Username","$Password");


# Check to make sure all info has been filled in
  if((!$Title)||(!$Keyword)||(!$Description)){

  wwwthreads::not_right("All of the required informatin has not been filled in.  Please try again.");

}


# Check to make sure the Keyword is available
  my $Keyword_q = $dbh -> quote($Keyword);
  my $query = qq!
    SELECT Keyword
    FROM Boards
    WHERE Keyword = $Keyword_q
  !;
  my $sth = $dbh -> prepare ($query) or die "Query syntax error: $DBI::errstr. Query: $query";
  $sth -> execute() or die "Can't execute query: $query. Reason: $DBI::errstr";

  my $rows = $sth -> rows;

  if( $rows > 0 ) {

    wwwthreads::not_right("That keyword is already being used.  Please try again.");

  }


# Get the board number 
  $query = qq!
    SELECT _seq FROM Boards
  !;
  $sth = $dbh -> prepare ($query) or die "Query syntax error: $DBI::errstr. Query: $query";
  $sth -> execute() or die "Can't execute query: $query. Reason: $DBI::errstr";
  my $Number = $sth -> fetchrow_array;


# Put the board into the database
  my $Title_q       = $dbh -> quote($Title);
  my $Description_q = $dbh -> quote($Description);
  my $Username_q    = $dbh -> quote($Username);
  my $HTML_q        = $dbh -> quote($HTML);
  my $Anonymous_q   = $dbh -> quote($Anonymous);
  my $Locked_q      = $dbh -> quote($Locked);

  $query = qq!
    INSERT INTO Boards
    (Title,Description,Keyword,Moderator,Total,Last,HTML,Anonymous,Locked,Number)
    VALUES ($Title_q, $Description_q, $Keyword_q,$Username_q,0,0,$HTML_q,$Anonymous_q,$Locked_q,$Number)
  !;
  
  $sth = $dbh -> prepare ($query) or die "Query syntax error: $DBI::errstr. Query: $query";
  $sth -> execute() or die "Can't execute query: $query. Reason: $DBI::errstr";


# Create the keyword's table
  $query = qq!
    CREATE TABLE $Keyword (
      Number UINT,
      Parent UINT,
      Main UINT,
      Posted UINT,
      Username CHAR(30),
      IP CHAR(20),
      Subject TEXT(100),
      Body TEXT(500)
  )
  !;

  $sth = $dbh -> prepare ($query) or die "Query syntax error: $DBI::errstr. Query: $query";
  $sth -> execute() or die "Can't execute query: $query. Reason: $DBI::errstr";


# Create the index on the Number
  $query = qq!
    CREATE INDEX Posts ON $Keyword(
      Number
    )
  !;

  $sth = $dbh -> prepare ($query) or die "Query syntax error: $DBI::errstr. Quer: $query";
  $sth -> execute() or die "Can't execute query: $query. Reason: $DBI::errstr";


# Create the index on the Parent
  $query = qq!
    CREATE INDEX Mother ON $Keyword(
      Parent
    )
  !;

  $sth = $dbh -> prepare ($query) or die "Query syntax error: $DBI::errstr. Query: $query";
  $sth -> execute() or die "Can't execute query: $query. Reason: $DBI::errstr";


# Create the Sequence
  $query = qq!
    CREATE SEQUENCE ON $Keyword STEP 1 VALUE 1
  !;
  $sth = $dbh -> prepare ($query) or die "Query syntax error: $DBI::errstr. Query: $query";
  $sth -> execute() or die "Can't execute query: $query. Reason: $DBI::errstr";


# Send them a confirmation
  wwwthreads::send_header("The new board has been created.");
  wwwthreads::table_header("The new board has been created.");

  print " The new board has been created and is now available for use by the users.  </P>";

  $sth -> finish;
  wwwthreads::send_footer();

