#!/usr/bin/perl 
###########################################################################
#    WWWThreads  - Interactive Discussion Forum
#    Rick Baker scream@screamingweb.com 
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
###########################################################################

  use wwwthreads qw(%config $dbh);  
  use strict;
  use DBI;

# Assign the variables
  my %FORM   = wwwthreads::get_input;
  my $action = $FORM{'action'};
  my $Board  = $FORM{'Board'};
  my $page   = $FORM{'page'};
  my $view   = $FORM{'view'};
  my $sb     = $FORM{'sb'};
  my $sort   = $FORM{'sort'};

# Grab a cookie
  my %cookie = wwwthreads::get_cookie();

# Set the default sort order
  if ( (!$sb) && ($cookie{'Sort'}) ) { 
    $sb = $cookie{'Sort'};
  } elsif( (!$sb) && (!$cookie{'Sort'}) ) {
    $sb = $wwwthreads::config{'sort'};
  }


# if $view isn't set then it gets set to the default view
  if ( (!$view) && ($cookie{'View'}) ) {
    $view = $cookie{'View'};  
  } elsif ( (!$view) && (!$cookie{'View'}) ) {
    $view = $wwwthreads::config{'threaded'};
  }


# if $page isn't set then it defaults to 0
  if (!$page){

    $page = 0;

  }


# If there is no specified action then just show the list of boards
  if(!$action){

    &show_list;

  }


# If action is 'list' then we give the list of posts for the selected board
  if($action eq "list"){

    &list_posts($Board,$page,$view,$sort,$sb);

  }


#End of this program
  wwwthreads::exit();


##########################################################################
## This subroutine shows the list of boards in the database
##########################################################################
  sub show_list{

  wwwthreads::db_connect();
  # Get the list of boards
    my $query = qq!
      SELECT Title,Description,Keyword,Moderator,Total,Last,HTML,Anonymous,Number,Locked
      FROM Boards
      ORDER BY Number
    !;
    my $sth = $dbh->prepare ($query) or die "Query syntax error: $DBI::errstr. Query: $query";
    $sth -> execute() or die "Can't execute query: $query. Reason: $DBI::errstr";
    my ($rows) = $sth -> rows;


  # Print the header
    wwwthreads::send_header("$wwwthreads::config{'title'}");


  # Give the list of boards`
    for(my $i=0;$i<$rows;$i++){

      my ($Title,$Description,$Keyword,$Moderator,$Total,$Last,$HTML,$Anonymous,$Number,$Locked) = $sth -> fetchrow_array;
    
    my $time = "";
    # If Last = 0 then this is a new board so set Last Post to New Board
      if (!$Last){

        $time = wwwthreads::convert_time($Last); 

      }else{

        $time = wwwthreads::convert_time($Last);

      }

      print qq~
      <table border=0 width=100% bgcolor=$wwwthreads::config{'tddark'} cellspacing=0>
      <tr><td width=60%>
      <a href="$wwwthreads::config{'cgiurl'}/wwwthreads.pl?action=list&Board=$Keyword">$Title</a></td><td width=10%>
      ~;


    # Check if the are new posts since they last visited
      my %cookie = wwwthreads::get_cookie;
      my $checker = $cookie{$Keyword};

      if($checker < $Last) {

        print "<font size=-1>$Total Posts <img src=\"$wwwthreads::config{'images'}/new.gif\">";

      } else {

        print "<font size=-1>$Total Posts";

      }
    
      print qq~ 
      </font></td><td width=30%>
      <font size=-1>Last Post: $time</font></td></tr></table>
      <table border=0 width=100% bgcolor=$wwwthreads::config{'tdlight'} cellspacing=0>
      <tr><td colspan=3>
      $Description
      <br><br>
      </td></tr>
      <tr><td width=33%>
      <i><font size=-1>Moderated by: $Moderator</font></i>
      ~;
 
      if ($Locked ne "Open") {

        print"</td><td width=67% colspan=2><i><font size=-1>This board is read only</font></i>";
        print"</td></tr></table>";

      } else {

        print"</td><td width=33%><i><font size=-1>HTML is $HTML</font></i>";
        print"</td><td width=34%><i><font size=-1>Anonymous Postings $Anonymous</font></i>";
        print"</td></tr></table>";

      }

    print "<table border=0 bgcolor=$wwwthreads::config{'tdalt'} width=100%><tr><td></td></tr></table>";
    }
  
  # Print the footer;
    $sth -> finish;
    wwwthreads::send_footer();
  }

############################################################################
## This subroutine lists the posts for the selected board
############################################################################
  sub list_posts{

    my($Board,$page,$view,$sort,$sb) = @_;

  # Connect to the database
    wwwthreads::db_connect();

    my $Board_q = $dbh -> quote($Board);

  # Grab the board info
    my $query = qq!
      SELECT Title,Locked,Last FROM Boards 
      WHERE Keyword = $Board_q
    !;
    my $sth = $dbh -> prepare($query) or die "Query syntax error: $DBI::errstr. Query: $query";
    $sth -> execute() or die "Can't execute query: $query. Reason: $DBI::errstr";
    my ($Title,$Locked,$Last) = $sth -> fetchrow_array;

  # Grab the cookie from them so we know what the last post they saw
  # and updates this to the current last post of the board
    my %cookie = wwwthreads::get_cookie;
    my $tname = "$Board.temp";	# The temporary cookie
    my $rname = "$Board.read";	# The temporary read cookie
    my $unread = "";

    if(!$cookie{$tname}) {

      my $oldlast = $cookie{$Board};
      wwwthreads::set_cookie($tname,"$oldlast",0);
      $unread = $oldlast;

      wwwthreads::set_cookie($Board,"$Last",1);

    } else {

      wwwthreads::set_cookie($Board,"$Last",1);

    }

    if (!$unread) {

      my %cookie = wwwthreads::get_cookie;
      $unread = $cookie{$tname};

    }

    my $read = $cookie{$rname};


  # Find out how many posts to show per page
    my $PostsPer = $cookie{'PostsPer'};
    $PostsPer ||= $wwwthreads::config{'postsperpage'};

  # Print the header
    wwwthreads::send_header($wwwthreads::config{'title'});

  # Show them how its sorted
    my %sort_title =  (
      1 => 'Descending Subject',
      2 => 'Ascending Subject',
      3 => 'Descending Username',
      4 => 'Ascending Username',
      5 => 'Descending Date',
      6 => 'Ascending Date'
    );
    my $sort_t = $sort_title{$sb};


  # print the title of the board
    print qq~
    <table border=0 width=100% bgcolor="$wwwthreads::config{'tddark'}" cellspacing=0>
    <tr><td align=center colspan=3><font size=+1><b>$Title</b></font>
    <br><font size=-1> ( Sorted by $sort_t ) </font></td></tr>
    <tr><td colspan=3 align=right>
    ~;

  # If the board is not open then we need to see if the user has logged in
  # and if they are an administrator before we give them a make post link.
    if($Locked ne "Open"){


    # Grab the cookie
      my %cookie = wwwthreads::get_cookie;


    # Assign the variables
      my $Username = $cookie{'Username'};
      my $Password = $cookie{'Password'};

      if (!$Username) { $Username = "Nb"; }
      my $Username_q = $dbh -> quote($Username);

    # Check if this user is an administrator
      my $query =qq!
        SELECT Status
        FROM Users
        WHERE Username = $Username_q
      !;
      $sth = $dbh -> prepare ($query) or die "Query syntax error: $DBI::errstr. Query: $query";
      $sth -> execute() or die "Can't execute query: $query. Reason: $DBI::errstr";
      my $Administrator = $sth -> fetchrow_array;

    # If status is administrator give them a New Post link
      if($Administrator eq "Administrator"){
 
        print "<a href=\"$wwwthreads::config{'cgiurl'}/newpost.pl?Board=$Board&page=$page&view=$view&sb=$sb\"><img src=\"$wwwthreads::config{'images'}/newpost.gif\" alt = \"Make a New Post\" border=0></a>";

      }


    # Otherwise we let them know that this board is locked
      else {
  
        print"<img src=\"$wwwthreads::config{'images'}/greynewpost.gif\">";

      }


  # Otherwise the board is open and new posts can be made
    } else {

      print "<a href=\"$wwwthreads::config{'cgiurl'}/newpost.pl?Board=$Board&page=$page&view=$view&sb=$sb\"><img src=\"$wwwthreads::config{'images'}/newpost.gif\" alt = \"Make a New Post\" border=0></a>";

    }


  my $sortsubject= 1;
  my $sortposter= 3;
  my $sortdate = 6;
  # Make sure we are giving the proper sort options
    if ($sb == 1) {
      $sortsubject = 2;
    } else {
      $sortsubject = 1;
    }
    if ($sb == 3){
      $sortposter = 4;
    } else {
      $sortposter = 3;
    }
    if ($sb ==5)  {
      $sortdate = 6;
    } else {
      $sortdate = 5;
    } 

  # Grab the total number of posts and figure out how we are sorting them
    my %sort_opt =  (
       1 => 'Subject DESC',
       2 => 'Subject ASC',
       3 => 'Username DESC',
       4 => 'Username ASC',
       5 => 'Posted DESC',
       6 => 'Posted ASC'
    );
    my $sort_by = $sort_opt{$sb};

    $query = qq!
      SELECT Number,Parent,Posted,Username,Subject,Main
      FROM $Board
      WHERE Number = Main
      ORDER BY $sort_by
    !;
    
    $sth = $dbh -> prepare ($query) or die "Query syntax error: $DBI::errstr. Query: $query";
    $sth -> execute() or die "Can't execute query: $query. Reason: $DBI::errstr";
    my $nums = $sth -> rows;


  # If $page is greater than 0 then we need a link to the previous page
    if($page > 0){

      my $prev = $page -1;
      print "<a href=\"$wwwthreads::config{'cgiurl'}/wwwthreads.pl?action=list&Board=$Board&page=$prev&view=$view&sb=$sb\"><img src=\"$wwwthreads::config{'images'}/previous.gif\" alt = \"Previous Page\" border=0></a>";

    } else {

      print "<img src =\"$wwwthreads::config{'images'}/greyprevious.gif\">";
 
    }


  # Give a link to the forum index
    print "<a href=\"$wwwthreads::config{'cgiurl'}/wwwthreads.pl\"><img src=\"$wwwthreads::config{'images'}/all.gif\" alt = \"Show all Forums\" border=0></a>";

 
  # If total parent posts is greater than $PostsPer*Page then we give them
  # a link to the next page
    
    if($nums > ($PostsPer*($page+1)) ){

      my $next = $page +1;
      print "<a href=\"$wwwthreads::config{'cgiurl'}/wwwthreads.pl?action=list&Board=$Board&page=$next&view=$view&sb=$sb\"><img src=\"$wwwthreads::config{'images'}/next.gif\" alt = \"Next Page\" border=0></a>";

    } else {

      print "<img src=\"$wwwthreads::config{'images'}/greynext.gif\">";

    }


  # If $postlist is "threaded" then we need to give them the proper
  # button to expand or collapse the threads
    if($wwwthreads::config{'postlist'} eq "threaded"){

      if($view eq "collapsed"){

        print"<a href=\"$wwwthreads::config{'cgiurl'}/wwwthreads.pl?action=list&Board=$Board&page=$page&view=expanded&sb=$sb\"><img src=\"$wwwthreads::config{'images'}/expand.gif\" alt = \"Expand Threads\" border=0></a>";
        print "<img src=\"$wwwthreads::config{'images'}/greycollapse.gif\">";
 
      } else {

        print "<img src=\"$wwwthreads::config{'images'}/greyexpand.gif\">";      
        print"<a href=\"$wwwthreads::config{'cgiurl'}/wwwthreads.pl?action=list&Board=$Board&page=$page&view=collapsed&sb=$sb\"><img src=\"$wwwthreads::config{'images'}/collapse.gif\" alt = \"Collapse Threads\" border=0></a>";

      }

    }  


  # Finish out the board header
    print qq~
    </td></tr>
    <tr><td colspan=3 bgcolor=$wwwthreads::config{'tdalt'}></td></tr>
    <tr><td bgcolor=$wwwthreads::config{'tdlight'} width=55%>
    <a href="$wwwthreads::config{'cgiurl'}/wwwthreads.pl?action=list&Board=$Board&page=$page&view=$view&sb=$sortsubject"><b>Subject</b></a></td>
    <td bgcolor=$wwwthreads::config{'tdlight'} width=20% align=left>
    <a href="$wwwthreads::config{'cgiurl'}/wwwthreads.pl?action=list&Board=$Board&page=$page&view=$view&sb=$sortposter"><b>Poster</b></a></td>
    <td bgcolor=$wwwthreads::config{'tdlight'} align=left>
    <a href="$wwwthreads::config{'cgiurl'}/wwwthreads.pl?action=list&Board=$Board&page=$page&view=$view&sb=$sortdate"><b>Posted on</b></a></td></tr>
    </table>
    ~;

  # List the posts for this page
    print "<table border=0 cellspacing=0 width=100%>";
    my $color = "$wwwthreads::config{'tddark'}";

    for(my $i=0;$i<$nums;$i++){

      my ($Number,$Parent,$Posted,$Username,$Subject,$Main) = $sth -> fetchrow_array;
      if( ($i >= ($PostsPer*$page) ) && ($i < ($PostsPer * ($page +1)) ) ){


      # convert the time
        my $time = wwwthreads::convert_time($Posted);


      # If view is expanded then we give the openfolder icon, if it is
      # collapsed then we give the closed folder.

        my $thisone = ",$Number,";
        my $folder = "closedfolder.gif";
        if (($view eq "expanded") && ($Posted <= $unread)) { 

          $folder = "openfolder.gif";

        } elsif (($view eq "collapsed") && ($Posted <= $unread)) {

          $folder = "closedfolder.gif";
  
        } elsif (($view eq "expanded") && ($Posted > $unread) && (!($read =~/$thisone/)) ) {

          $folder = "newopenfolder.gif";

        } elsif (($view eq "collapsed") && ($Posted > $unread) && (!($read =~/$thisone/)) ){

          $folder = "newclosedfolder.gif";

        } elsif (($view eq "collapsed") && ($Posted > $unread) && ($read =~/$thisone/) ) {

          $folder = "closedfolder.gif";

        } elsif (($view eq "expanded") && ($Posted > $unread) && ($read =~/$thisone/) ) {

          $folder = "openfolder.gif";

        }

        print "<tr><td bgcolor=$color>";
        print "<img src=\"$wwwthreads::config{'images'}/$folder\" hspace=5>";
        print "<a href=\"$wwwthreads::config{'cgiurl'}/showpost.pl?Board=$Board&Number=$Number&page=$page&view=$view&sb=$sb\">$Subject</a>";


      # If we are doing flat posts or the view is collapsed we need to list
      # the number of replies
        if (($wwwthreads::config{'postlist'} eq "flat") || ($view eq "collapsed")) {

          $query =qq!
            SELECT Parent
            FROM $Board
            WHERE Main = $Number
            AND Number <> $Number
          !;
          my $sti = $dbh -> prepare($query) or die "Query syntax error: $DBI::errstr. Query: $query";
          $sti -> execute() or die "Can't execute query: $query. Reason: $DBI::errstr";
          my $replies = $sti -> rows;
          if ($replies < 1) { $replies = 0; }

            print "<font size=-1> ($replies)</font>";

        }
   
        if ($Username eq "Anonymous"){

          print"</td><td width=20% bgcolor=$color>Anonymous</td>";

        }else{

          print "</td><td width=20% bgcolor=$color><a href=\"$wwwthreads::config{'cgiurl'}/showprofile.pl?User=$Username&Board=$Board&what=wwwthreads&page=$page&view=$view&sb=$sb\">$Username</a></td>";

        }

        print "<td width=25% bgcolor=$color>$time</td></tr>";

      # alternate the colors
        if ($color eq "$wwwthreads::config{'tdlight'}") {
          $color = "$wwwthreads::config{'tddark'}";
        } else {
          $color = "$wwwthreads::config{'tdlight'}";
        }
        
      
      # If we are not doing flat posts and the view is expanded then we
      # will list all the replies out in a threaded format.
        if (($wwwthreads::config{'postlist'} ne "flat") && ($view eq "expanded") ){
          $query = qq!
            SELECT Number
            FROM $Board
            WHERE Parent = $Parent
          !; 
          my $sta = $dbh -> prepare ($query) or die "Query syntax error: $DBI::errstr. Query: $query";
          $sta -> execute() or die "Can't execute query: $query. Reason: $DBI::errstr";
          my $followups = $sta -> rows;
          if ($followups > 0) {
            $sta -> finish;
            $color = &show_replies($Number,$Board,0,$unread,$color,$page,$view,$sb,$sort);

          }

        }

      }

    }

    print "</table>";

  # Print the footer
    $sth -> finish;
    wwwthreads::send_footer;

  }

###########################################################################
# show replies - list the replies here if we are doing threaded, expanded
###########################################################################

sub show_replies {

    my ($Mother,$Board,$indent,$unread,$color,$page,$view,$sb,$sort) = @_;
    my $query = qq!
      SELECT Number,Parent,Posted,Username,Subject
      FROM $Board
      WHERE Parent = $Mother
      ORDER BY Number Desc
    !;
    my $sth = $dbh -> prepare ($query) or die "Query syntax error: $DBI::errstr. Query: $query";
    $sth -> execute() or die "Can't execute query: $query. Reason: $DBI::errstr";
    my ($followups) = $sth -> rows;
  
    $indent++;
    
    for(my $x=0;$x<$followups;$x++){

      my ($Number,$Parent,$Posted,$Username,$Subject) = $sth -> fetchrow_array;

      print "<tr><td bgcolor=$color>";
      for (my $y=0;$y<$indent;$y++){
    
        print "<img src=\"$wwwthreads::config{'images'}/blank.gif\">";

      }

      my $time = wwwthreads::convert_time($Posted);

      my $folder = "closedfolder.gif";
      if($Posted <= $unread){

        $folder = "closedfolder.gif";

      } else {

        $folder = "newclosedfolder.gif";

      }

      print"<img src=\"$wwwthreads::config{'images'}/$folder\" hspace=5><a href=\"$wwwthreads::config{'cgiurl'}/showpost.pl?Board=$Board&Number=$Number&page=$page&view=$view&sb=$sb#Post$Number\">$Subject</a>";

        if ($Username eq "Anonymous"){

          print"</td><td width=20% bgcolor=$color>Anonymous</td>";

        }else{

          my $encoded = wwwthreads::url_decode ($Username);
          print "</td><td width=20% bgcolor=$color><a href=\"$wwwthreads::config{'cgiurl'}/showprofile.pl?User=$encoded&Board=$Board&what=wwwthreads&page=$page&view=$view&sb=$sb\">$Username</a></td>";
        }

      print "<td width=25% bgcolor=$color>$time</td></tr>";

    # alternate the colors
      if ($color eq "$wwwthreads::config{'tdlight'}") {
        $color = "$wwwthreads::config{'tddark'}";
      } else {
        $color = "$wwwthreads::config{'tdlight'}";
      }

      $query =qq!
        SELECT Number
        FROM $Board
        WHERE Parent = $Number
      !;

      my $sta = $dbh -> prepare ($query) or die "Query syntax error: $DBI::errstr. Query: $query";
      $sta -> execute() or die "Can't execute query: $query. Reason: $DBI::errstr";
      my $follows = $sta -> rows;
      if ($follows > 0) {
        $sta -> finish;
        $color = &show_replies($Number,$Board,$indent,$unread,$color,$page,$view,$sb,$sort);

      }

    }

    $indent--;
    $sth -> finish;
    return $color;
  }

