###########################################################################
#    WWWThreads  - Interactive Discussion Forum
#    Rick Baker scream@screamingweb.com 
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
###########################################################################
package wwwthreads;

require Exporter;
@ISA = qw (Exporter);
@EXPORT = qw(@EXPORT_OK);

@EXPORT_OK = qw(%config $dbh);

use strict;
use DBI;

use vars qw(%config $dbh); 

###########################################################################
# User defined variables
###########################################################################
# Url to the cgi-bin containing the WWWThreads package
    $config{'cgiurl'}   = "http://www.gamernews.net/cgi-bin/wwwthreads";

  # Url containing images - Currently unused
    $config{'images'}   = "http://www.gamernews.net/wwwthreads/images";

  # Path containing the .include files
    $config{'includes'} = "/home/httpd/cgi-bin/wwwthreads/includes";

  # Name of the server running sql - (yourhost.com or localhost)
    $config{'dbserver'} = "localhost";

  # Name of the database you created for the sql tables
    $config{'dbname'}   = "WWWThreads";

  # Name of the DBI driver you are using
    $config{'dbdriver'} = "mSQL";

  # Name of the database user to connect as.
    $config{'dbuser'}   = '';

  # Password of the database user.
    $config{'dbpass'}   = '';

  # Use persistant database connections. 
  # Note: This is still extreme beta.  Each http child process will keep
  # a connection open to the database.  This may or may not work properly
  # Set this to 1 to use persistant or set it to 0 to not use it.
    $config{'persistant'} = 0;

  # Path to sendmail
    $config{'mailprog'} = "/usr/sbin/sendmail";

  # URL of the server the program is running from
    $config{'referer'}  = "http://www.gamernews.net";

  # Default listing to flat or threaded.  
  # If you set this to "flat" then when you view a board, a list of all 
  # parent posts will be shown, and when you click on a post to read, all 
  # replies to this post will be listed on one page.
  # If you set this to "threaded" then when you view a board a thread list
  # of all parent posts and replies will be shown.
    $config{'postlist'} = "threaded";

  # Default sort order
  # 1 - Descending Subject
  # 2 - Ascending Subject
  # 3 - Descdending Poster
  # 4 - Ascending Poster
  # 5 - Descending Date
  # 6 - Ascending Date
    $config{'sort'} = 5;

  # Default threaded lists to collapsed or expanded
  # If your $postlist = "threaded" then you can set this variable to either
  # "collapsed" or "expanded".
  # If you set it to "collapsed" then only the parent posts will be shown
  # and you will have to click on the message to view the replies.
  # If you set it to "expanded" then parent posts and replies will be shown
  # on the main page.
    $config{'threaded'} = "collapsed";

  # Default number of posts per page - This only counts parent posts
    $config{'postsperpage'} = "10";

  # When persistent cookies will expire
    $config{'cexp'} = 'Thu, 31-Dec-2020 23:00:00 GMT';

  # 1 = Use a background image : 0 = don't use background image
    $config{'useimage'} = 0;                
                                      
  # If useimage=1, name of image to be used 
    $config{'backimage'} = "back.gif";      
                          
  # Color of webpage backgrounds                 
    $config{'bodybg'} = "#FFFFFF";

  # Table Header colors
    $config{'tableheader'} = "#DDDDDD";

  # Dark table colors
    $config{'tddark'} = "#DDDDDD";

  # Light table colors
    $config{'tdlight'} = "#EEEEEE";

  # Alternate table colors
    $config{'tdalt'} = "#0000080";

  # Text color
    $config{'textcol'} = "black";

  # Link color
    $config{'linkcol'} = "blue";

  # Visited link color
    $config{'linkvis'} = "purple";

  # Active link color
    $config{'linkactive'} ="red";

  # Title of site
    $config{'title'} = "WWWThreads";

  # Your home page
    $config{'homeurl'} = "http://www.gamernews.net";

  # Text for your homepage link
    $config{'urltitle'} = "GamerNews.Net";

  # Your email address
    $config{'emailaddy'} = "scream\@screamingweb.com";

  # Text for your email address
    $config{'emailtitle'} = "Contact Me";

  # If your time needs to be adjusted, in hours (2,1,0,-1,-2)
    $config{'adjusttime'} = 0;

  # Show the ip address of the posters
    $config{'showip'} = 1;

  # Let people have replies mailed to them directly
  # Setting this to 1 allows it, setting it to 0 disallows it
    $config{'usemail'} = 1;

  # Default to having all replies mailed to the users
  # Default to not having all replies mailed to the users


###########################################################################
# Start of the main subroutines
###########################################################################


###########################################################################
# Get_input - Parses form input and returns it in a hash.
# Useage: my %in = &get_input;
###########################################################################
sub get_input {

  my $input = '';
  my %in    = ();
  ($ENV{'REQUEST_METHOD'} eq "POST") and  read(STDIN, $input, $ENV{'CONTENT_LENGTH'});
  ($ENV{'REQUEST_METHOD'} eq "GET")  and  ($input = $ENV{'QUERY_STRING'});

  my @pairs = split(/&/, $input);
  foreach (@pairs) {
    my ($name, $value) = split /=/;
  # Un-Webify plus signs and %-encoding
    $value =~ tr/+/ /;
    $value =~ s/%([a-fA-F0-9][a-fA-F0-9])/pack("C", hex($1))/eg;
    $value =~ s/<!--(.|\n)*-->//g;
    $value =~ s/\n/<br>/g;
    $value =~ s/\r//g;
    $in{$name} = $value;
  }
  return %in;

}


###########################################################################
# url_encode: Encodes a string for printing in a URL.
# Usage: my $encoded = &url_encode ($string);
###########################################################################
sub url_encode {

  my $input = shift;
  $input =~s/([^a-zA-Z0-9_.-])/uc sprintf("%%%02x",ord($1))/eg;
  return $input

}

 
###########################################################################
# Url_decode: Decodes a string that's been encoded in a URL.
# Usage: my $string = &url_decode ($encoded);
###########################################################################
sub url_decode {

  my $input = shift;
  $input =~ tr/+/ /;
  $input =~ s/%([0-9a-fA-F]{2})/pack("c",hex($1))/ge;
  return $input;

}


###########################################################################
# check_refer: Checks for a valid referer
###########################################################################
sub check_refer {

  my $valid = 0;
  ($ENV{'HTTP_REFERER'} =~ /$wwwthreads::config{'referer'}/i) and $valid++;
    
  $valid or wwwthreads::not_right ("The host you are trying to send the input from is not a valid host.");

}


###########################################################################
# get_date: returns a date in seconds
###########################################################################
sub get_date {

  my ($currtime)=time();
  my ($adjust)=$currtime+($wwwthreads::config{'adjusttime'}*3600);
  return $adjust;

}


###########################################################################
# convert_time: converts the time to a useable format
###########################################################################
sub convert_time {

  my $time = shift; 
  $time = localtime($time);
  $time = `date --date=\"$time\" +\"%D %R\"`;
  return $time;

}


###########################################################################
# Connects to the database.
###########################################################################
sub db_connect{

  defined $dbh or
    ($dbh = DBI -> connect("DBI:$wwwthreads::config{'dbdriver'}:$wwwthreads::config{'dbname'}:$wwwthreads::config{'dbserver'}", "$wwwthreads::config{'dbuser'}", "$wwwthreads::config{'dbpass'}")
    or die "Can't connect to sql server.  Reason: $DBI::errstr");

}


###########################################################################
# Disconnects from the database
###########################################################################
sub db_disconnect{

  defined $dbh and
    ($dbh->disconnect or die "Can't disconnect from database. Reason: $DBI::errstr");
  undef $dbh;

}


###########################################################################
# validate_admin. Checks to make sure the user is an admin or moderator
# Usage: &validate_admin ($user, $pass)
###########################################################################
sub validate_admin {

  my ($username, $password) = @_;

  wwwthreads::db_connect();    # Make sure we are connected.
  my $username_q = $dbh->quote($username);
  my $password_q = $dbh->quote($password);
  my $query = qq!
    SELECT * 
    FROM Users
    WHERE  Username = $username_q  AND
    Password = $password_q  AND
    Status   = 'Administrator'
  !;
  my $sth = $dbh->prepare ($query) or die "Query syntax error: $DBI::errstr. Que
ry: $query";
  $sth->execute() or die "Query execute error: $DBI::errstr. Query: $query";

  if ($sth->rows < 1) {

    $sth->finish;
    wwwthreads::not_right ("Sorry, but you are not a valid administrator.");

  }

  $sth->finish;

}


###########################################################################
# Get_cookie - Gets all the cookies and returns them in a hash
# Usage: my %cookie = &get_cookie;
###########################################################################
sub get_cookie {

  my @cstuff = @_;
  my ($cookie, $value, $char, %cookie);

  my @Cdec = ('\+', '\%3A\%3A', '\%3D', '\%2C', '\%25', '\%2B', '\%26','\%3B');
  my %Cdec = ('\+',' ','\%3A\%3A','::','\%3D','=','\%2C',',','\%25','%','\%2B','+','\%26','&','\%3B',';');

  if ($ENV{'HTTP_COOKIE'}) {

    foreach (split(/; /,$ENV{'HTTP_COOKIE'})) {

    ($cookie,$value) = split(/=/);

      foreach $char (@Cdec) {

        $cookie =~ s/$char/$Cdec{$char}/g;
        $value =~ s/$char/$Cdec{$char}/g;

      }

      $cookie{$cookie} = $value;

    }

  }

  return %cookie;

}

###########################################################################
# set_cookie - Sets a cookie.
# Usage : set_cookie('Name',"Value",?) ? = 0 or 1. 0 = temp, 1 = permanent
###########################################################################
sub set_cookie {

  my @cookie = @_;
  my ($cookie, $value, $type, $char);    
  my @Cenc = ('\;','\&','\+','\%','\,','\=','\:\:','\s');
  my %Cenc = ('\;','%3B','\&','%26','\+','%2B','\%','%25','\,','%2C','\=','%3D','\:\:','%3A%3A','\s','+');

  while( ($cookie,$value,$type) = @cookie ) {

    foreach $char (@Cenc) {

      $cookie =~ s/$char/$Cenc{$char}/g;
      $value =~ s/$char/$Cenc{$char}/g;

    }

    if ($type == 0) {

      print "Set-Cookie: $cookie=$value; expires= ;path=/; \n";

    }
    elsif ($type == 1) {

      print "Set-Cookie: $cookie=$value; expires=$wwwthreads::$config{'cexp'}; path=/; \n";

    }

    shift(@cookie); shift(@cookie); shift(@cookie);

  }

}


###########################################################################
# Figure out how we should exit
###########################################################################
sub exit{

  if(exists $ENV{MOD_PERL}) {
    Apache::exit(0) 
  }
  else {
    CORE::exit(0);
  }

}

###########################################################################
# The following subs define the looks and feel of the program
###########################################################################

###########################################################################
# Prints out the beginning HTML.  Pass this $title and $refresh.
###########################################################################
sub send_header{

  my ($title, $refresh) = @_;
  print "Content-type: text/html\n\n";
  print "<HTML><HEAD>";
  defined $refresh and print $refresh;
  print "<TITLE>$title</TITLE></HEAD>";
  $wwwthreads::config{'useimage'} ?
     print qq~<body background = "$wwwthreads::config{'backimage'}" text = "$wwwthreads::config{'textcol'}" link = "$wwwthreads::config{'linkcol'}" vlink = "$wwwthreads::config{'linkvis'}" alink = "$wwwthreads::config{'linkactive'}">~ :
     print qq~<body bgcolor = "$wwwthreads::config{'bodybg'}" text = "$wwwthreads::config{'textcol'}" link = "$wwwthreads::config{'linkcol'}" vlink = "$wwwthreads::config{'linkvis'}" alink = "$wwwthreads::config{'linkactive'}">~;

# Include the header file.
  open(FILE,"$wwwthreads::config{'includes'}/header.include") or &cgi_error("Unable to open header file: $wwwthreads::config{'includes'}/header.include. Reason: $!");
  while(<FILE>) {

    print;

  }
  close(FILE);

# Include the menu file.
  print "<table border=0 width=100%><tr><td valign=top width=10%>";
  open(FILE,"$wwwthreads::config{'includes'}/menu.include") or &cgi_error("Unable to open menu file: $wwwthreads::config{'includes'}/menu.include. Reason: $!");
  while (<FILE>) {

    s/_PARSER_/$wwwthreads::config{'cgiurl'}/g;
    print $_;

  }

  print "</td><td valign=top>";

}


###########################################################################
# table_header . Prints the table header bar that most pages use.
# Usage: &table_header("Header 1");
###########################################################################
sub table_header {

  my $header = shift;
  print qq~
  <table border=0 width=100% bgcolor=$wwwthreads::config{'tableheader'}>
    <tr><td>
    <font size=+2><b>
    $header
    </b></font>
    </td></tr>
  </table>
  <p>
  ~;

}

 
###########################################################################
# send_footer - Prints out the page footer
###########################################################################
sub send_footer{

  print qq~
  </td></tr></table>
  <P align=center>
  <a href="$wwwthreads::config{'homeurl'}">$wwwthreads::config{'urltitle'}</a>
  &nbsp;&nbsp;&nbsp;
  &nbsp;&nbsp;&nbsp;
  &nbsp;&nbsp;&nbsp;
  <a href="mailto:$wwwthreads::config{'emailaddy'}">$wwwthreads::config{'emailtitle'}</a>
  </P>
  <P align=center>
  <B>
  <FONT SIZE=-1>
  Forums Powered By
  <br>
  <A HREF="http://www.gamernews.net/wwwthreads">WWWThreads Version 3.0a</A>
  </B>
  </FONT>
  </P>        
  ~;

  open(FILE,"$wwwthreads::config{'includes'}/footer.include") or &cgi_error("Unable to open footer file: $wwwthreads::config{'includes'}/footer.include. Reason: $!");
  while (<FILE>){

    print;

  } 

  close(FILE);
  print "</BODY></HTML>";

  if($wwwthreads::config{'persistant'} == 0) {

    wwwthreads::db_disconnect;

  }

}


###########################################################################
# not_right- User did something wrong.
###########################################################################
sub not_right{

  my $error = shift;
  &send_header ("There was a problem!");
  &table_header ("We encountered a problem!");
  print qq~
    The server encountered an error and cannot complete your request.  The error reported was:
    </p><p>
    <i>
    $error
    </i>
    </p><p>
    If you feel you this error is a problem with the server then please contact <a href="mailto:$wwwthreads::config{'emailaddy'}">$wwwthreads::config{'emailaddy'}</a> and let us know what exactly went wrong so it can be fixed as soon as possible.  Otherwise, please use your back button to return to the previous page and try again.
    </p>
  ~;
  &send_footer;   # Send a footer back to the browser
  wwwthreads::exit();
}

###########################################################################
# cgi_error - System did something wrong
###########################################################################
sub cgi_error {

  my $error = shift;
  print "Content-type: text/html\n\n";
  print "System Error: $error";
  #wwwthreads::exit();
  die;
}


###########################################################################
# start_page - Logs the user on.
###########################################################################
sub start_page {
  my ($Username, $Password, $Bypass, $info) = @_;
  my $sth = '';
  my $query = '';
  my $rows = '';


# Connect to db and get total posts and last log on.
  wwwthreads::db_connect();
  my $username_q = $dbh->quote($Username);
  my $password_q = $dbh->quote($Password);
  $query = qq~
    SELECT Totalposts, Laston, Password, Sort, Display, View, PostsPer
    FROM   Users
    WHERE  Username = $username_q 
  ~;
  $sth = $dbh->prepare ($query) or die "Query syntax error: $DBI::errstr. Que
ry: $query";
  $sth->execute() or die "Query execute error: $DBI::errstr. Query: $query";

# Make sure logs on ok..    
  if ($sth->rows < 1) {
    $sth->finish;
    wwwthreads::not_right ("We couldn't find you in our database!");
    }
  my ($posts, $laston, $pass, $sort, $display, $view, $postsper) = $sth->fetchrow_array;

# Check the password
  if ($pass ne $Password) {
    $sth->finish;
    wwwthreads::not_right ("The password you gave does not match the password in our database for that Username.");
  }

# Set a cookie.    
  wwwthreads::set_cookie('Username',"$Username",0,'Password',"$Password",0,'Sort',"$sort",1,'Display',"$display",1,'View',"$view",1,'PostsPer',"$postsper",1);

# Update last log on time.
  my $date = &get_date;
  $query = qq~
    UPDATE Users 
    SET    Laston = $date 
    WHERE  Username = $username_q
  ~;
  my $rc = $dbh ->do($query) or die "Query execute error: $DBI::errstr. Query: $
query";

# Now give them their page.
  wwwthreads::send_header ("Welcome, $Username");
  wwwthreads::table_header ("Welcome, $Username");
  my $lastlogon = &convert_time($laston);
  print qq~
    <a href="$wwwthreads::config{'cgiurl'}/editprofile.pl"><img src="$wwwthreads::config{'images'}/editprofile.gif" alt="Edit Profile" border=0></a>
    <a href="$wwwthreads::config{'cgiurl'}/sendpriv.pl"><img src="$wwwthreads::config{'images'}/sendprivate.gif" alt="Send private message" border=0></a>
    <a href="$wwwthreads::config{'cgiurl'}/logout.pl"><img src="$wwwthreads::config{'images'}/logout.gif" alt="Logout" border=0></a> 
    <p>
    Your last logon was at: <i>$lastlogon</i><br>
    Your total posts: <i>$posts</i><br><br>
    </p>
  ~;

  print "<p>$info</p>";

# Get any private messages.
  $query = qq~
    SELECT Read, Subject, Sender, When, Number
    FROM   Messages
    WHERE  Username = $username_q
    ORDER BY When DESC
  ~;
  $sth = $dbh->prepare ($query) or die "Query syntax error: $DBI::errstr. Que
ry: $query";
  $sth->execute() or die "Query execute error: $DBI::errstr. Query: $query";

  $rows = $sth->rows;
  if ($rows < 1) {
    print "You have no private messages.";
  }
  else {
    print qq~
      You have the following private messages.
      <table border=0 width=100%>
      <tr><td bgcolor=$wwwthreads::config{'tddark'} width=5%>&nbsp;</td>\n<td bgcolor=$wwwthreads::config{'tddark'} width=45%>Subject</td>\n<td bgcolor=$wwwthreads::config{'tddark'} width=20%>From</td>\n<td bgcolor=$wwwthreads::config{'tddark'} width=25%>Received</td>\n<td bgcolor=$wwwthreads::config{'tddark'} width=5%>Delete</td></tr>
      <form method=post action = \"$wwwthreads::config{'cgiurl'}/delete.pl\">
      <input type=hidden name=total value = $rows>
    ~;
    my ($read, $subject, $sender, $when, $number, $i);
      $i = 0;
    while (($read, $subject, $sender, $when, $number) = $sth->fetchrow_array) { 
      my $senton = &convert_time($when);
      if ($read eq 'N') {
        print "<tr><td bgcolor=$wwwthreads::config{'tdlight'}>N</td>\n";
      }
      elsif ($read eq 'R') {
        print "<tr><td bgcolor=$wwwthreads::config{'tdlight'}>R</td>\n";
      }
      else {
        print "<tr><td bgcolor=$wwwthreads::config{'tdlight'}>&nbsp;</td>\n";
      }
      print qq~
      <td bgcolor=$wwwthreads::config{'tdlight'}><a href="$wwwthreads::config{'cgiurl'}/viewmessage.pl?message=$number">$subject</a></td>
      <td bgcolor=$wwwthreads::config{'tdlight'}>$sender</td>
      <td bgcolor=$wwwthreads::config{'tdlight'}>$senton</td>
      <td bgcolor=$wwwthreads::config{'tdlight'}><input type=checkbox name=$i value=$number></td></tr>
      ~;
      $i++;
    }
    print "</table>";
    print "<input type = submit name = option value = \"Delete checked\">\n";
    print "</form>\n"; 
  }
  $sth->finish;
  &send_footer;
}


#########################################################################
# Admin_start
#########################################################################

sub admin_start{

  my ($Username, $Password) = @_;


# Validate the logon
  wwwthreads::validate_admin("$Username","$Password");


# Give the user a cookie, (Mmmm.. cookies)
  wwwthreads::set_cookie("Username","$Username",0,"Password","$Password",0);


# They made it this far, so it is a valid user.  Give them the
# Administration page. 
  wwwthreads::send_header("Welcome, $Username");
  wwwthreads::table_header("Welcome, $Username");

  print qq~
    As an Administrator the following options are available to you
    <p>
    <a href="$wwwthreads::config{'cgiurl'}/admin/createboard.pl">
    Create Board
    </a>
  
  ~;


# Send the footer
  wwwthreads::send_footer();

}

